/****************************************************************************
 
   trm - the TRM digital finger print utilty 
  
   Copyright (C) 2002 Robert Kaye

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version. 
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details. 
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software 
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.

   $Id: main.cpp,v 1.5 2002/09/26 22:22:01 robert Exp $

 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <musicbrainz/mb_c.h>
#include "config.h"
#include "trm.h"

#ifndef WIN32
#define stricmp(s,d) strcasecmp(s,d)
#else
#include <winsock.h>
#endif

void main(int argc, char *argv[])
{
    char    sig[37], *ptr;
#ifdef WIN32
    WSADATA sGawdIHateMicrosoft;
#endif
    int     isMP3 = 0, isOv = 0, isWav = 0, index = 1;

    if (argc < 2)
    {
        fprintf(stderr,"usage: trm [-m|-o|-w] <mp3 | ogg/vorbis | wav file>\n");
        exit(-1);
    }

#ifdef WIN32
    WSAStartup(0x0002,  &sGawdIHateMicrosoft);
#endif

    for(;;)
    {
        if (stricmp(argv[index], "-m") == 0)
        {
           isMP3 = 1;
           index++;
        }
        else
        if (stricmp(argv[index], "-o") == 0)
        {
           isOv = 1;
           index++;
        }
        else
        if (stricmp(argv[index], "-w") == 0)
        {
           isWav = 1;
           index++;
        }
        else 
           break;
    }

	if (access(argv[index], 0))
	{
		fprintf(stderr, "Cannot open file %s\n", argv[index]);
		exit(-1);
	}

    if (!isMP3 && !isOv && !isWav)
    {
        ptr = strrchr(argv[index], '.');
        if (ptr && stricmp(ptr, ".mp3") == 0)
           isMP3 = 1;

        if (ptr && stricmp(ptr, ".ogg") == 0)
           isOv = 1;

        if (ptr && stricmp(ptr, ".wav") == 0)
           isWav = 1;

        if (isMP3 == 0 && isOv == 0 && isWav == 0)
        {
            fprintf(stderr, "error: cannot determine audio file type.\n");
            fprintf(stderr, "       use -m, -o, -w to set the file type.\n");
            exit(-1); 
        }                             
    }

    if ((isMP3 && isOv) || (isMP3 && isWav) || (isOv && isWav))
    {
        fprintf(stderr, "error: both mp3 and ogg/vorbis file types specied.\n");
        fprintf(stderr, "       make up your mind and pick one!\n");
        exit(-1); 
    }

    if (isMP3)
    {
#ifdef HAVE_LIBMAD
        int ret;

        ret = MP3_generateTRM(argv[1], sig);
        if (ret == 0)
            printf("%s\n", sig);
        else
            fprintf(stderr, "error: failed to generate TRM id\n");
#else
        fprintf(stderr, "error: mp3 support not built into application\n");
#endif
    }
    if (isOv)
    {
#ifdef HAVE_OGGVORBIS
        int ret;

        ret = OggVorbis_generateTRM(argv[1], sig);
        if (ret == 0)
            printf("%s\n", sig);
        else
            fprintf(stderr, "error: failed to generate TRM id\n");
#else
        fprintf(stderr, "error: ogg/vorbis support not built into application\n");
#endif
    }
    if (isWav)
    {
        int ret;

        ret = Wav_generateTRM(argv[1], sig);
        if (ret == 0)
            printf("%s\n", sig);
        else
            fprintf(stderr, "error: failed to generate TRM id\n");
    }

#ifdef WIN32
    WSACleanup();
#endif
}
